#! /bin/bash
# To use this script, copy it to directory /etc/keepalived/scripts
# and add one or more of the following to the keepalived configuration
# file in the global_defs section:
#     notify_fifo /etc/keepalived/scripts/notify_fifo
#     vrrp_notify_fifo /etc/keepalived/scripts/vrrp_notify_fifo
#     lvs_notify_fifo /etc/keepalived/scripts/lvs_notify_fifo
# This script will then need to be executed, passing it the name of the
# fifo.
#
# As an alternative to executing this script manually, add one or more of
# the following in the global_defs section of the config:
#     notify_fifo_script /etc/keepalived/scripts/sample_notify_fifo.sh
#     vrrp_notify_fifo_script /etc/keepalived/scripts/sample_notify_fifo.sh
#     lvs_notify_fifo_script /etc/keepalived/scripts/sample_notify_fifo.sh
# If run this way, Keepalived will terminate the script with SIGTERM when
# it exits.

PROC=$(basename $0)
STATE_MASTER="01"
STATE_BACKUP="02"
LOG_FILE="/var/log/tcr/services.log"
STATE_FILE="/tmp/vrrp_state"
CREATED_FIFO=0
IS_STOPPED=0
IS_DURING_READ=0

TCR_ID=`grep -w tcr_id /opt/tcr/tcr.conf | cut -d\" -f 2`

function log_msg () {
    NOW=$(date +"%b %d %T.%4N")
    echo "$NOW $HOSTNAME $PROC [$$]: $1" >>  $LOG_FILE;
}

function save_state () {
  filename=$STATE_FILE"_"$1
  echo -n "$STR_STATE" > $filename;
}

FIFO=$1
[[ -z $FIFO ]] && log_msg "A FIFO name must be specified" && exit 1

start_shutdown()
{
    # When keepalived terminates, it sends a TERM signal to this script before
    #  sending the fifo notifies. We catch the SIGTERM here, break from the blocking "read" and exit gracefully.
    # Future messages will be handled by the new script

    if [ $IS_STOPPED -eq 0 ] ; then
      log_msg "Received termination signal, starting graceful termination"
      
      if [ $CREATED_FIFO -ne 1 ] ; then
        if [ $IS_DURING_READ -eq 1 ] ; then
          log_msg "FIFO is handled by keepalived, exiting..."  
          exit 0
        else
          log_msg "Finishing state handling before starting termination"
          IS_STOPPED=1
          return
        fi
      fi
      
      log_msg "FIFO is handled locally, attempting exit from blocking read from FIFO"

      # Send a dummy message through the FIFO to exit from the blocking "read" call
      echo "SIGTERM" > $FIFO
    fi

    IS_STOPPED=1
}

trap "{ [[ $CREATED_FIFO -eq 1 ]] && rm -f $FIFO; exit 0; }" HUP INT QUIT USR1 USR2 PIPE ALRM
trap start_shutdown TERM

if [[ ! -p $FIFO ]]; then
    mkfifo $FIFO
    if [[ $? -eq 0 ]]; then
        CREATED_FIFO=1
        log_msg "Created FIFO successfully"
    else
        log_msg "Unable to create fifo $FIFO"
        exit 1
    fi
fi

# If keepalived terminates, the FIFO will be closed, so
# read the FIFO in a loop. It keepalived hasn't opened the
# FIFO, the script will be blocked until it has been opened.
while [ $IS_STOPPED -eq 0 ]
do
   [[ ! -p $FIFO ]] && log_msg "FIFO $FIFO missing" && exit 1

   while [ $IS_STOPPED -eq 0 ] ; do
      IS_DURING_READ=1
      read line
      rc=$?
      IS_DURING_READ=0
      
      # Preserve original behavior of exiting inner-loop when "read line" fails
      if [ $rc -ne 0 ] ; then
        break 
      fi
      
      # Terminate after reciving SIGTERM
      if [[ $line == "SIGTERM" ]] ; then
        log_msg "Exited blocking read call from FIFO"
        if [ $CREATED_FIFO -eq 1 ] ; then
          log_msg "Deleting FIFO and exiting"
          rm -f $FIFO 
          exit 0
        fi
        
        break
      fi

      set $line
      TYPE=$1
      VRRP_INST=${2//\"/}
      STR_STATE=$3
      PRIORITY=$4
      CUR_STATE=$STATE_BACKUP
      if [ "$STR_STATE" == "STOP" ]; then
         log_msg "Deleting VRRP state file"
         unlink $STATE_FILE
      fi

      if [ "$STR_STATE" != "BACKUP" -a  "$STR_STATE" != "MASTER" -a "$STR_STATE" != "FAULT" ]; then
          log_msg "Ignoring state '$STR_STATE'"
          continue
      fi

      msg_type="\x26\x00" # msgtyp_vrrp_state hex
      msg_sender="\x56\x52\x52\x50\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00" # VRRP
      msg_sender_pid="\x00\x00\x00\x00"
      msg_reciever="\x54\x43\x52\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00" # TCR
      msg_timestamp=$(printf '%016llx' $(date +%s%3N))

      if [ -e "$TCR_ID" -o -e "$STR_STATE" ]; then
          log_msg "Missing parameters"
          continue
      fi

      if [ "$STR_STATE" == "MASTER" ]; then
          CUR_STATE=$STATE_MASTER;
      fi

      site_id=$(echo $VRRP_INST | awk -F_ '{print $1}')
      if [[ $VRRP_INST == *_temp ]]; then
          site_id+="_temp"
      fi
      inst_name_len=${#site_id}
      padding_len=$(( 68 - inst_name_len ))
      padding=$(seq 1 1 $padding_len | xargs printf '\\x00%.0s')
      msg_data="$site_id$padding\x$CUR_STATE"
      log_msg "About to send '$CUR_STATE' message to TCR ..."

      echo -n -e "$msg_type$msg_sender$msg_sender_pid$msg_reciever$msg_data$msg_timestamp" | nc -w0 -uU /opt/tcr/sock/$TCR_ID/tcr && log_msg "Sent successfully !"

      save_state "$site_id"

      if [ "$STR_STATE" == "BACKUP" ]; then
          log_msg "Removing vrrp interfaces $(ip addr show | grep ':vrrp_[0-9]' | awk '{print $2}')"
          ip addr show | grep ":vrrp_[0-9]" | awk '{print $2 " " $NF}' | while read -r ip label ; do ip addr del $ip dev $label ; done
      fi
   done < $FIFO
done

log_msg "Exiting script"